package com.gic.auth.dto;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;
import java.util.Date;

/**
 * @author guojx
 * @date 2019/7/16 6:49 PM
 */
public class UserDTO implements Serializable{
    private static final long serialVersionUID = 7296135586565827660L;

    public interface SaveUserValid {
    }

    public interface EditUserValid {
    }

    public interface UserQoValid {
    }




    /**
     * 用户id
     */
    @NotNull(message = "用户ID不能为空", groups = {EditUserValid.class})
    private Integer userId;

    /**
     * 用户名
     */
    @NotBlank(message = "用户名不能为空", groups = {SaveUserValid.class, EditUserValid.class, UserQoValid.class})
    private String userName;

    /**
     * 手机号码
     */
    @NotBlank(message = "手机号不能为空", groups = {SaveUserValid.class, EditUserValid.class, UserQoValid.class})
    @Pattern(message = "手机号不规范", regexp = "1[3|4|5|7|8][0-9]\\d{8}", groups = {SaveUserValid.class, EditUserValid.class, UserQoValid.class})
    private String phoneNumber;

    /**
     * 密码
     */
    @NotBlank(message = "密码不能为空", groups = {SaveUserValid.class})
    private String password;

    /**
     * 1：自动生成 2：自定义密码
     */
    @NotNull(message = "登陆密码类型不能为空", groups = {UserQoValid.class})
    private Integer passwordType;

    /**
     * 确认密码
     */
    private String confirmPassword;

    /**
     * 是否是超级管理员
     */
    @NotNull(message = "用户类型不能为空", groups = {SaveUserValid.class})
    private Integer superAdmin;

    /**
     * 状态，0有效，1无效
     */
    private Integer deleteFlag;

    /**
     *
     */
    private Date createTime;

    /**
     *
     */
    private Date updateTime;

    /**
     *
     */
    @NotNull(message = "企业ID不能为空", groups = {SaveUserValid.class, EditUserValid.class})
    private Integer enterpriseId;

    /**
     * 国际区号，如中国 86
     */
    private String phoneAreaCode;

    /**
     * 受审组，可以有多个，用英文逗号隔开
     */
    private String userGroupIds;

    /**
     * 登陆账号类型 1:运营 0:默认GIC
     */
    private Integer loginType;

    /**
     * 账号类型 1:永久账号 2:临时账号
     */
    private Integer accountType;

    /**
     * 临时账号到期时间
     */
    private Date accountDueDate;

    /**
     * 临时账号到期时间 前端
     */
    private String accountDueDateStr;

    /**
     * 职位 1:CRM经理、2:商城运营经理、3:IT经理 子管理员必填
     */
    private Integer userPosition;

    /**
     * 用户角色，可以多选，用英文逗号隔开
     */
    private String userRoleIds;
    /**
     * 用户资源组授权，可以多选，用英文逗号隔开
     */
    private String userResourceIds;
    /**
     * 1：是编辑操作，并且没修改过密码 2：与1相反
     */
    private Integer operPasswordType;
    /**
     * 账号分组，可以多选，用英文逗号隔开
     */
    private String accountGroupIds;

    /***
     * 营销是否触达  1是0否
     */
    private Integer markingFlag;

    /**
     * 微信ID
     */
    private String openId;

    public Integer getUserId() {
        return userId;
    }

    public void setUserId(Integer userId) {
        this.userId = userId;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getPhoneNumber() {
        return phoneNumber;
    }

    public void setPhoneNumber(String phoneNumber) {
        this.phoneNumber = phoneNumber;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public Integer getSuperAdmin() {
        return superAdmin;
    }

    public void setSuperAdmin(Integer superAdmin) {
        this.superAdmin = superAdmin;
    }

    public Integer getDeleteFlag() {
        return deleteFlag;
    }

    public UserDTO setDeleteFlag(Integer deleteFlag) {
        this.deleteFlag = deleteFlag;
        return this;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Date getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Date updateTime) {
        this.updateTime = updateTime;
    }

    public Integer getEnterpriseId() {
        return enterpriseId;
    }

    public void setEnterpriseId(Integer enterpriseId) {
        this.enterpriseId = enterpriseId;
    }

    public String getPhoneAreaCode() {
        return phoneAreaCode;
    }

    public void setPhoneAreaCode(String phoneAreaCode) {
        this.phoneAreaCode = phoneAreaCode;
    }

    public Integer getPasswordType() {
        return passwordType;
    }

    public void setPasswordType(Integer passwordType) {
        this.passwordType = passwordType;
    }

    public String getConfirmPassword() {
        return confirmPassword;
    }

    public void setConfirmPassword(String confirmPassword) {
        this.confirmPassword = confirmPassword;
    }

    public String getUserGroupIds() {
        return userGroupIds;
    }

    public void setUserGroupIds(String userGroupIds) {
        this.userGroupIds = userGroupIds;
    }

    public Integer getLoginType() {
        return loginType;
    }

    public void setLoginType(Integer loginType) {
        this.loginType = loginType;
    }

    public String getUserRoleIds() {
        return userRoleIds;
    }

    public void setUserRoleIds(String userRoleIds) {
        this.userRoleIds = userRoleIds;
    }

    public String getUserResourceIds() {
        return userResourceIds;
    }

    public void setUserResourceIds(String userResourceIds) {
        this.userResourceIds = userResourceIds;
    }

    public Integer getOperPasswordType() {
        return operPasswordType;
    }

    public void setOperPasswordType(Integer operPasswordType) {
        this.operPasswordType = operPasswordType;
    }

    public String getAccountGroupIds() {
        return accountGroupIds;
    }

    public void setAccountGroupIds(String accountGroupIds) {
        this.accountGroupIds = accountGroupIds;
    }

    public Integer getAccountType() {
        return accountType;
    }

    public UserDTO setAccountType(Integer accountType) {
        this.accountType = accountType;
        return this;
    }

    public Date getAccountDueDate() {
        return accountDueDate;
    }

    public UserDTO setAccountDueDate(Date accountDueDate) {
        this.accountDueDate = accountDueDate;
        return this;
    }

    public Integer getUserPosition() {
        return userPosition;
    }

    public UserDTO setUserPosition(Integer userPosition) {
        this.userPosition = userPosition;
        return this;
    }

    public String getAccountDueDateStr() {
        return accountDueDateStr;
    }

    public UserDTO setAccountDueDateStr(String accountDueDateStr) {
        this.accountDueDateStr = accountDueDateStr;
        return this;
    }

    public Integer getMarkingFlag() {
        return markingFlag;
    }

    public void setMarkingFlag(Integer markingFlag) {
        this.markingFlag = markingFlag;
    }

    public String getOpenId() {
        return openId;
    }

    public UserDTO setOpenId(String openId) {
        this.openId = openId;
        return this;
    }

    @Override
    public String toString() {
        return "UserDTO{" +
                "userId=" + userId +
                ", userName='" + userName + '\'' +
                ", phoneNumber='" + phoneNumber + '\'' +
                ", password='" + password + '\'' +
                ", passwordType=" + passwordType +
                ", confirmPassword='" + confirmPassword + '\'' +
                ", superAdmin=" + superAdmin +
                ", deleteFlag=" + deleteFlag +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", enterpriseId=" + enterpriseId +
                ", phoneAreaCode='" + phoneAreaCode + '\'' +
                ", userGroupIds='" + userGroupIds + '\'' +
                ", loginType=" + loginType +
                ", accountType=" + accountType +
                ", accountDueDate=" + accountDueDate +
                ", accountDueDateStr='" + accountDueDateStr + '\'' +
                ", userPosition=" + userPosition +
                ", userRoleIds='" + userRoleIds + '\'' +
                ", userResourceIds='" + userResourceIds + '\'' +
                ", operPasswordType=" + operPasswordType +
                ", accountGroupIds='" + accountGroupIds + '\'' +
                ", markingFlag=" + markingFlag +
                ", openId='" + openId + '\'' +
                '}';
    }
}
