package com.gic.auth.service;

import java.util.List;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AccountGroupDTO;
import com.gic.auth.dto.AccountGroupMemberDTO;

/**
 * 账号分组
 * @ClassName: AccountGroupApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/10/28 4:09 PM 
 */
public interface AccountGroupApiService {

    /**
     * 账号分组新增
     * @Title: save 
     * @Description:
      * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> save(AccountGroupDTO dto);

    /**
     * 编辑账号分组
     * @Title: update 
     * @Description:
      * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> update(AccountGroupDTO dto);

    /**
     * 查询账号分组列表数据，有组员数量
     * @Title: listAccountGroupByEnterpriseId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AccountGroupDTO>> 
      */
    ServiceResponse<List<AccountGroupDTO>> listAccountGroupByEnterpriseId(Integer enterpriseId);

    /**
     * 查询账号分组列表数据，无数量
     * @Title: listNoMemberByEnterpriseId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AccountGroupDTO>> 
      */
    ServiceResponse<List<AccountGroupDTO>> listNoMemberByEnterpriseId(Integer enterpriseId);

    /**
     * 删除账号分组，并且删除分组内的组员关系
     * @Title: deleteByAccountGroupId 
     * @Description:
      * @author guojuxing
     * @param accountGroupId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.AccountGroupDTO> 
      */
    ServiceResponse<AccountGroupDTO> deleteByAccountGroupId(Integer accountGroupId);

    /**
     * 目标排序
     * @Title: sort 
     * @Description:
      * @author guojuxing
     * @param accountGroupId
     * @param sort 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.AccountGroupDTO> 
      */
    ServiceResponse<AccountGroupDTO> sort(Integer accountGroupId, Integer sort);

    /**
     * 根据分组ID查询分组内的组员，给会员组使用的接口
     * @Title: listAccountGroupMemberByAccountGroupId 
     * @Description:
      * @author guojuxing
     * @param accountGroupIdList 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AccountGroupMemberDTO>> 
      */
    ServiceResponse<List<AccountGroupMemberDTO>> listAccountGroupMemberByAccountGroupId(List<Integer> accountGroupIdList);

    /**
     * 查询不在对应账号分组内的所有组员
     * @Title: listNotInAccountGroupByAccountGroupId 
     * @Description:
      * @author guojuxing
     * @param accountGroupId 
     * @param search
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserListDTO>> 
      */
    ServiceResponse<List<AccountGroupMemberDTO>> listNotInAccountGroupByAccountGroupId(Integer accountGroupId,
            String search);


    /**
     * 查询商户所有组员
     * @Title: listAllMember 
     * @Description:
      * @author guojuxing
     * @param search 
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AccountGroupMemberDTO>> 
      */
    ServiceResponse<List<AccountGroupMemberDTO>> listAllMember(String search, Integer enterpriseId);

    /**
     * 查询在对应账号分组内的所有组员
     * @Title: listNotInAccountGroupByAccountGroupId 
     * @Description:
      * @author guojuxing
     * @param accountGroupId 
     * @param search
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.UserListDTO>> 
      */
    ServiceResponse<List<AccountGroupMemberDTO>> listInAccountGroupByAccountGroupId(Integer accountGroupId, String search);

    /**
     * 批量保存分组组员，必须有分组ID、管理员/协作人ID、类型
     * @Title: saveAccountGroupRelFetch 
     * @Description:
      * @author guojuxing
     * @param list 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 返回日志操作对象内容 
      */
    ServiceResponse<String> saveAccountGroupRelFetch(List<AccountGroupMemberDTO> list);

    /**
     * 管理员批量分组
     * @Title: groupingOfUser 
     * @Description:
      * @author guojuxing
     * @param userIdList
     * @param accountGroupId 
     * @param enterpriseId
     * @return  com.gic.api.base.commons.ServiceResponse<java.langgroupingOfUser.String>  返回XXX管理员分组到XXX分组的字符串
      */
    ServiceResponse<String> groupingOfUser(List<Integer> userIdList, List<Integer> accountGroupId, Integer enterpriseId);

}
