package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.*;
import com.gic.commons.annotation.SkipNamingCheck;

import java.util.List;

/**
 * 资源组
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/8 4:36 PM 
 */
@SkipNamingCheck
public interface ResourceGroupApiService {

    /**
     * 新增
     * @param resourceGroup
     * @return
     */
    ServiceResponse<Integer> saveResourceGroup(ResourceGroupDTO resourceGroup);

    /**
     * 编辑资源组
     * @param resourceGroup
     * @return
     */
    ServiceResponse<String> editResourceGroup(ResourceGroupDTO resourceGroup);

    /**
     * 授权给联合商户的通知 syncResourceAuthChangeMQ
     * @param ownEnterpriseId 自有商户ID
     * @param unionEnterpriseId 联合商户ID
     * @param newResourceGroup
     * @param oldResourceGroup
     * @param resourceGroupChangeType 1 : 初始化 2：资源组变更, 默认1
     * @param logParams TransmitDataDTO transmitDataDTO = TransmitDataHolder.get();
     * @return
     */
    ServiceResponse<Void> syncResourceAuthChangeMQ(Integer ownEnterpriseId, Integer unionEnterpriseId,
                                                   ResourceGroupDTO newResourceGroup, ResourceGroupDTO oldResourceGroup, Integer resourceGroupChangeType, String logParams);

    /**
     * 分页查询
     * @param name 资源组名称
     * @param enterpriseId
     * @param pageNum
     * @param pageSize
     * @return
     */
    ServiceResponse<Page<ResourceGroupDTO>> pageResourceGroup(String name, Integer enterpriseId, Integer pageNum, Integer pageSize);

    /**
     * 查询资源组列表
     * @param name
     * @param enterpriseId
     * @return
     */
    ServiceResponse<List<ResourceGroupDTO>> listResourceGroup(String name, Integer enterpriseId);

    /**
     * 查询详情
     * @param resourceGroupId
     * @return
     */
    ServiceResponse<ResourceGroupDTO> getResourceGroup(Integer resourceGroupId);

    /**
     * 查询列表
     * @param resourceGroupIdList
     * @return
     */
    ServiceResponse<List<ResourceGroupDTO>> listResourceGroup(List<Integer> resourceGroupIdList);

    /**
     * 删除资源组
     * @param resourceGroupId
     * @return
     */
    ServiceResponse<String> deleteResourceGroup(Integer resourceGroupId);

    /**
     * 根据资源组数据查询，用于登录时存入session
     * @param userId
     * @return
     */
    ServiceResponse<ResourceGroupDataDTO> getResourceGroupByUserId(Integer userId);

    /**
     * 管理员的门店资源查询
     * @param userId
     * @return
     */
    ServiceResponse<StoreResourceDTO> getStoreResourceByUserId(Integer userId);

    /**
     * 管理员的用户域配置信息
     * @param userId
     * @return
     */
    ServiceResponse<CustomUserAreaDTO> getCustomUserByUserId(Integer userId);

    /**
     * 管理员的商品资源
     * @param userId
     * @return
     */
    ServiceResponse<GoodsResourceDTO> getGoodsResourceByUserId(Integer userId);
}
