package com.gic.auth.config;

import javax.validation.ConstraintViolationException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.context.annotation.Configuration;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;

/**
 * api层错误全部统一返回
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/9 9:46 AM 
 */
@Aspect
@Configuration
public class ExceptionResultAop {

    private Logger logger = LogManager.getLogger(ExceptionResultAop.class);

    /**
     * 环绕通知
     *
     * @param  joinPoint
     * @return
     * @throws Throwable
     */
    @Around(value = "execution(* com.gic.*.service.outer.impl..*.*(..))")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
        try {
            // 执行当前方法
            return joinPoint.proceed();
        } catch (Throwable throwable) {
            logger.warn("拦截器错误", throwable);
            return handlerException(throwable);
        }
    }

    private ServiceResponse handlerException(Throwable e) {
        ServiceResponse response = new ServiceResponse();
        if (e instanceof CommonException) {
            response.setCode(((CommonException) e).getErrorCode());
        } else if (e instanceof ConstraintViolationException) {
            response.setCode(ErrorCode.SYSTEM_ERROR.getCode());
        } else {
            response.setCode(ErrorCode.SYSTEM_ERROR.getCode());

        }
        response.setMessage(e.getMessage());
        return response;
    }
}