package com.gic.auth.service;

import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserListDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.UserListQO;
import com.github.pagehelper.Page;

import java.util.List;

/**
 * @author guojx
 * @date 2019/7/16 6:50 PM
 */
public interface UserService {

    /**
     * 新增
     * @param userDTO
     * @return
     */
    int saveUser(UserDTO userDTO);

    /**
     * 新增运营实施管理员
     * @Title: saveOperationUser 
     * @Description:
      * @author guojuxing
     * @param userDTO 
     * @return  int 
      */
    int saveOperationUser(UserDTO userDTO);

    /**
     * editUser
     * @Title: editUser
     * @Description:
     * @author zhiwj
     * @param userDTO
     * @return void
     * @throws
     */
    void editUser(UserDTO userDTO);

    /**
     * 根据主键获取数据
     * @param userId
     * @return
     */
    TabSysUser getUserById(Integer userId);

    /**
     * 商户下openId不能重复
     * @param enterpriseId
     * @param openid
     * @param userId
     * @return
     */
    boolean validOpenidIsRepeat(Integer enterpriseId, String openid, Integer userId);

    /**
     * 根据openId查询
     * @param openId
     * @param enterpriseId
     * @return
     */
    TabSysUser getUserByOpenId(String openId, Integer enterpriseId);

    /**
     * 手机号码是否重复
     * @param userId
     * @param phone
     * @param enterpriseId
     * @return
     */
    boolean isPhoneRepeat(Integer userId, String phone, Integer enterpriseId);

    /**
     * 手机号码是否重复
     * @Title: isPhoneRepeat 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param phone 手机号码
     * @param enterpriseId 商户ID
     * @param loginType  管理员类别
     * @return  boolean 
      */
    boolean isPhoneRepeat(Integer userId, String phone, Integer enterpriseId, Integer loginType);

    /**
     * getUserByEnterpriseId
     * @Title: getUserByEnterpriseId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.auth.entity.TabSysUser
     * @throws
     */
    TabSysUser getUserByEnterpriseId(Integer enterpriseId);

    /**
     * 查询超管信息
     * @param enterpriseIdList
     * @return
     */
    List<TabSysUser> listUserByEnterpriseId(List<Integer> enterpriseIdList);

    /**
     * listAllUserByPhoneNumber
     * @Title: listAllUserByPhoneNumber
     * @Description:
     * @author zhiwj
     * @param phoneNumber
     * @param nationCode
     * @return java.util.List<com.gic.auth.entity.TabSysUser>
     * @throws
     */
    List<TabSysUser> listAllUserByPhoneNumber(String phoneNumber, String nationCode);

    /**
     * listUserByPhoneNumberAndEnterprise
     * @Title: listUserByPhoneNumberAndEnterprise
     * @Description:
     * @author zhiwj
     * @param phoneNumber
     * @param nationCode
     * @param enterpriseId
     * @return java.util.List<com.gic.auth.entity.TabSysUser>
     * @throws
     */
    List<TabSysUser> listUserByPhoneNumberAndEnterprise(String phoneNumber, String nationCode, Integer enterpriseId);

    /**
     * login
     * @Title: login
     * @Description:
     * @author zhiwj
     * @param nationCode
     * @param phoneNumber
     * @param enterpriseId
     * @param password
     * @return com.gic.auth.entity.TabSysUser
     * @throws
     */
    TabSysUser login(String nationCode, String phoneNumber, Integer enterpriseId, String password);

    /**
     * 查询管理员列表数据
     * @Title: pageUser 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.github.pagehelper.Page 
      */
    Page<UserListDTO> pageUser(UserListQO params);

    /**
     * 符合转移对象的管理员列表
     * @param params
     * @return
     */
    Page<TabSysUser> pageUserForTransfer(UserListQO params);

    /**
     * 管理员列表
     * @Title: listUser
     * @Description:
     * @author zhiwj
     * @param params
     * @return com.github.pagehelper.Page<com.gic.auth.dto.UserListDTO>
     * @throws
     */
    List<UserListDTO> listUser(UserListQO params);

    /**
     * 逻辑删除
     * @Title: delete 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  void 
      */
    void delete(Integer userId);

    /**
     * getAllCheckValue
     * @Title: getAllCheckValue
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param userIdList
     * @return java.lang.Integer
     * @throws
     */
    Integer getAllCheckValue(Integer enterpriseId, List<Integer> userIdList);

    /**
     * validUserIsNotExist
     * @Title: validUserIsNotExist
     * @Description:
     * @author zhiwj
     * @param userIdList
     * @return boolean
     * @throws
     */
    boolean validUserIsNotExist(List<Integer> userIdList);

    /**
     * 用户ID list查询用户信息
     * @Title: listUserByIdList 
     * @Description:
      * @author guojuxing
     * @param userIdList 
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<TabSysUser> listUserByIdList(List<Integer> userIdList);

    /**
     * 查询不在范围内的用户
     * @Title: listUserNotInIdList 
     * @Description:
      * @author guojuxing
     * @param userIdList 
     * @param search
     * @param enterpriseId
     * @return  java.util.List<com.gic.auth.entity.TabSysUser> 
      */
    List<TabSysUser> listUserNotInIdList(List<Integer> userIdList, String search, Integer enterpriseId);

    /**
     * 查询所有的用户，gic管理员/运营实施管理员
     * @Title: listAllUser 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  java.util.List<java.lang.Integer> 
      */
    List<Integer> listAllUser(Integer enterpriseId);

    /**
     * 查询用户
     * @param enterpriseId
     * @param isGic 0：gic 1:运营
     * @return
     */
    List<TabSysUser> listGicUser(Integer enterpriseId, Integer isGic);
}
