package com.gic.auth.service.impl;

import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.dao.mapper.TabSysAccountGroupMapper;
import com.gic.auth.dto.AccountGroupDTO;
import com.gic.auth.entity.TabSysAccountGroup;
import com.gic.auth.service.AccountGroupService;
import com.gic.commons.util.EntityUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
/**
 * 
 * @ClassName: AccountGroupServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:36
 */
@Service("AccountGroupService")
public class AccountGroupServiceImpl implements AccountGroupService{
    @Autowired
    private TabSysAccountGroupMapper tabSysAccountGroupMapper;
    @Override
    public int save(AccountGroupDTO dto) {
        dto.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        TabSysAccountGroup record = EntityUtil.changeEntityNew(TabSysAccountGroup.class, dto);
        tabSysAccountGroupMapper.insert(record);
        return record.getAccountGroupId();
    }

    @Override
    public int update(AccountGroupDTO dto) {
        dto.setUpdateTime(new Date());
        TabSysAccountGroup record = EntityUtil.changeEntityNew(TabSysAccountGroup.class, dto);
        return tabSysAccountGroupMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int getMaxSortValueByEnterpriseId(Integer enterpriseId) {
        return tabSysAccountGroupMapper.getMaxSortValueByEnterpriseId(enterpriseId);
    }

    @Override
    public boolean isAccountGroupNameRepeat(Integer enterpriseId, Integer accountGroupId, String accountGroupName) {
        int count = tabSysAccountGroupMapper.countByAccountGroupName(enterpriseId, accountGroupId, accountGroupName);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public TabSysAccountGroup getById(Integer accountGroupId) {
        return tabSysAccountGroupMapper.selectByPrimaryKey(accountGroupId);
    }

    @Override
    public List<TabSysAccountGroup> listByIdList(List<Integer> accountGroupIdList) {
        return tabSysAccountGroupMapper.listByIdList(accountGroupIdList);
    }

    @Override
    public List<TabSysAccountGroup> listAccountGroupByEnterpriseId(Integer enterpriseId) {
        return tabSysAccountGroupMapper.listAccountGroupByEnterpriseId(enterpriseId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int updateStatusByAccountGroupId(Integer accountGroupId) {
        TabSysAccountGroup record = new TabSysAccountGroup();
        record.setAccountGroupId(accountGroupId);
        record.setDeleteFlag(DeleteFlagConstants.DELETE_STATUS);
        return tabSysAccountGroupMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public void setSort(TabSysAccountGroup record, Integer sortValue) {
        List<TabSysAccountGroup> list = tabSysAccountGroupMapper.listAccountGroupByEnterpriseId(record.getEnterpriseId());
        Integer fromSortValue = record.getSort();
        if (fromSortValue > sortValue) {
            //向上拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysAccountGroup temp = list.get(i);
                //如果大于sortValue，都需要降低排序值,往后推
                boolean isNeedDown = temp.getSort() >= sortValue && temp.getSort() < fromSortValue;
                if (isNeedDown) {
                    updateSort(list.get(i + 1).getSort(), temp.getAccountGroupId());
                }
            }
        } else if (fromSortValue < sortValue) {
            //向下拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysAccountGroup temp = list.get(i);
                boolean isNeedUp = temp.getSort() <= sortValue && temp.getSort() > fromSortValue;
                if (isNeedUp) {
                    updateSort(list.get(i - 1).getSort(), temp.getAccountGroupId());
                }
            }
        }
        updateSort(sortValue, record.getAccountGroupId());
    }

    private void updateSort(Integer sort, Integer accountGroupId) {
        TabSysAccountGroup record = new TabSysAccountGroup();
        record.setAccountGroupId(accountGroupId);
        record.setSort(sort);
        tabSysAccountGroupMapper.updateByPrimaryKeySelective(record);
    }
}
