package com.gic.auth.service.impl;

import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.dao.mapper.TabAuditedGroupUserRelMapper;
import com.gic.auth.dto.AuditedGroupDTO;
import com.gic.auth.dto.AuditedGroupUserRelDTO;
import com.gic.auth.entity.TabAuditedGroupUserRel;
import com.gic.auth.service.AuditedGroupUserRelService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-18 11:31
 */
@Service
public class AuditedGroupUserRelServiceImpl implements AuditedGroupUserRelService {

    @Autowired
    private TabAuditedGroupUserRelMapper tabAuditedGroupUserRelMapper;

    @Override
    public Integer addOrUpdateRel(AuditedGroupDTO auditedGroupDTO) {
        // 把所有auditedGroup的关联status设置为0
        tabAuditedGroupUserRelMapper.delByAuditedGroupId(auditedGroupDTO.getAuditedGroupId());
        // 插入不存在的关联
        for (Integer userId : auditedGroupDTO.getUserIdList()) {
            TabAuditedGroupUserRel auditedGroupUserRel = new TabAuditedGroupUserRel();
            auditedGroupUserRel.setEnterpriseId(auditedGroupDTO.getEnterpriseId());
            auditedGroupUserRel.setAuditedGroupId(auditedGroupDTO.getAuditedGroupId());
            auditedGroupUserRel.setUserId(userId);
            auditedGroupUserRel.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
            auditedGroupUserRel.setCreateTime(new Date());
            tabAuditedGroupUserRelMapper.insertSelectiveByNotExist(auditedGroupUserRel);
        }
        // 把所有userId关联status都设置为1
        return tabAuditedGroupUserRelMapper.updateStatusByUserIds(auditedGroupDTO.getEnterpriseId(), auditedGroupDTO.getAuditedGroupId(), auditedGroupDTO.getUserIdList());
    }

    @Override
    public int delRel(Integer auditedGroupId) {
        return tabAuditedGroupUserRelMapper.delByAuditedGroupId(auditedGroupId);
    }

    @Override
    public Map<Integer, Integer> getUserCountByAuditedGroup(List<Integer> auditedGroupIds) {
        Map<Integer, Map<Integer, Long>> mapList = tabAuditedGroupUserRelMapper.getUserCountByAuditedGroup(auditedGroupIds);
        Map<Integer, Integer> resultMap = new HashMap<>(auditedGroupIds.size());
        for (Map.Entry<Integer, Map<Integer, Long>> entry : mapList.entrySet()) {
            Integer auditedGroupId = entry.getKey();
            resultMap.put(auditedGroupId, entry.getValue().get("userCount").intValue());
        }
        return resultMap;
    }

    @Override
    public List<TabAuditedGroupUserRel> listAuditedGroup(Integer auditedGroupId) {
        AuditedGroupUserRelDTO groupUserRel = new AuditedGroupUserRelDTO();
        groupUserRel.setAuditedGroupId(auditedGroupId);
        return this.tabAuditedGroupUserRelMapper.listAuditedGroup(groupUserRel);
    }

    @Override
    public List<TabAuditedGroupUserRel> getUserIdListByAuditedGroup(List<Integer> auditedGroupIds) {
        AuditedGroupUserRelDTO groupUserRel = new AuditedGroupUserRelDTO();
        groupUserRel.setAuditedGroupIdList(auditedGroupIds);
        return this.tabAuditedGroupUserRelMapper.listAuditedGroup(groupUserRel);
    }

    @Override
    public List<Integer> listAuditedGroupByUserId(Integer enterpriseId, Integer userId) {
        return tabAuditedGroupUserRelMapper.listAuditedGroupByUserId(enterpriseId, userId);
    }

    @Override
    public void deleteByUserId(Integer userId) {
        tabAuditedGroupUserRelMapper.deleteByUserId(userId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void setUserAuditedGroup(Integer userId, Integer enterpriseId, List<Integer> auditedGroupId) {
        //删除该用户的所有关联
        deleteByUserId(userId);
        if (CollectionUtils.isEmpty(auditedGroupId)) {
            return ;
        }
        //建立关联
        Date now = new Date();
        for (Integer auditedGroupIdTemp : auditedGroupId) {
            TabAuditedGroupUserRel record = new TabAuditedGroupUserRel();
            record.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
            record.setAuditedGroupId(auditedGroupIdTemp);
            record.setCreateTime(now);
            record.setUpdateTime(now);
            record.setUserId(userId);
            record.setEnterpriseId(enterpriseId);
            tabAuditedGroupUserRelMapper.insert(record);
        }
    }
}
