package com.gic.auth.service.impl;

import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.dao.mapper.TabSysResourceGroupMapper;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.entity.TabSysResourceGroup;
import com.gic.auth.service.ResourceGroupService;
import com.gic.commons.util.EntityUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * 资源组
 * @ClassName:  
 * @Description:
 * @author guojuxing   
 * @date 2021/1/8 4:41 PM  
 */
@Service("resourceGroupService")
public class ResourceGroupServiceImpl implements ResourceGroupService{
    @Autowired
    private TabSysResourceGroupMapper tabSysResourceGroupMapper;

    @Override
    public Integer saveResourceGroup(ResourceGroupDTO resourceGroup) {
        TabSysResourceGroup record = EntityUtil.changeEntityNew(TabSysResourceGroup.class, resourceGroup);
        record.setUpdateTime(new Date());
        record.setCreateTime(new Date());
        record.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
        tabSysResourceGroupMapper.insert(record);
        return record.getResourceGroupId();
    }

    @Override
    public int editResourceGroup(ResourceGroupDTO resourceGroup) {
        TabSysResourceGroup record = EntityUtil.changeEntityNew(TabSysResourceGroup.class, resourceGroup);
        record.setUpdateTime(new Date());
        return tabSysResourceGroupMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public Page<TabSysResourceGroup> pageResourceGroup(String name, Integer enterpriseId, Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<TabSysResourceGroup> list = tabSysResourceGroupMapper.pageResourceGroup(name, enterpriseId);
        return (Page<TabSysResourceGroup>) list;
    }

    @Override
    public List<TabSysResourceGroup> listResourceGroup(String name, Integer enterpriseId) {
        return tabSysResourceGroupMapper.pageResourceGroup(name, enterpriseId);
    }

    @Override
    public TabSysResourceGroup getResourceGroup(Integer resourceGroupId) {
        return tabSysResourceGroupMapper.getResourceGroup(resourceGroupId);
    }

    @Override
    public List<TabSysResourceGroup> listResourceGroup(List<Integer> resourceGroupIdList) {
        if (CollectionUtils.isEmpty(resourceGroupIdList)) {
            return null;
        }
        return tabSysResourceGroupMapper.listResourceGroupByIds(resourceGroupIdList);
    }

    @Override
    public int deleteResourceGroup(Integer resourceGroupId) {
        return tabSysResourceGroupMapper.deleteResourceGroup(resourceGroupId);
    }

    @Override
    public boolean isRepeatResourceGroupName(Integer resourceGroupId, String resourceGroupName, Integer enterpriseId) {
        int count = tabSysResourceGroupMapper.countRepeatResourceGroupName(resourceGroupId, resourceGroupName, enterpriseId);
        if (count > 0) {
            return true;
        }
        return false;
    }
}
