package com.gic.auth.service.outer.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.enterprise.service.EnterpriseLicensePatternApiService;
import com.gic.platform.dto.TransmitDataDTO;
import com.gic.platform.manager.TransmitDataHolder;
import com.gic.store.service.StoreAuthorizationApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.*;
import com.gic.auth.entity.TabSysResourceGroup;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.ResourceGroupService;
import com.gic.auth.service.UserResourceService;
import com.gic.auth.utils.resourcegroup.ResourceGroupUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import com.gic.store.service.UnionStoreChangeApiService;

/**
 * 资源组
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/8 4:38 PM 
 */
@Service("resourceGroupApiService")
public class ResourceGroupApiServiceImpl implements ResourceGroupApiService{
    private static Logger logger = LogManager.getLogger(ResourceGroupApiServiceImpl.class);
    @Autowired
    private ResourceGroupService resourceGroupService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;
    @Autowired
    private UnionStoreChangeApiService unionStoreChangeApiService;
    @Autowired
    private WmStoreApiService wmStoreApiService;

    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;
    @Autowired
    private EnterpriseLicensePatternApiService enterpriseLicensePatternApiService;

    @Override
    public ServiceResponse<Integer> saveResourceGroup(ResourceGroupDTO resourceGroup) {
        validParam(resourceGroup);
        Integer resourceGroupId = resourceGroupService.saveResourceGroup(resourceGroup);
        return ServiceResponse.success(resourceGroupId);
    }

    @Override
    public ServiceResponse<String> editResourceGroup(ResourceGroupDTO resourceGroup) {
        validParam(resourceGroup);
        //是否联合授权，如果是
        UnionEnterpriseAuthDTO unionEnterpriseAuthDTO = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuthByResourceGroupId(resourceGroup.getResourceGroupId()).getResult();
        //有效的才会重新授权。如果是已解除、待解除、解除中，不再重新授权
        if (unionEnterpriseAuthDTO != null && unionEnterpriseAuthDTO.getStatusFlag() == 1) {
            //验证码验证
            boolean isUnionEnterpriseAuthResourceGroup = "hasValid".endsWith(resourceGroup.getAuthCode());
            if (!isUnionEnterpriseAuthResourceGroup) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "验证码未通过");
            }

            if (!enterpriseLicensePatternApiService.validUserLicenseEnough(resourceGroup.getResourceGroupId(),
                    unionEnterpriseAuthDTO.getUnionEnterpriseId()).getResult()) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户的用户域license不足");
            }

            ResourceGroupDTO oldResourceGroup = EntityUtil.changeEntityNew(ResourceGroupDTO.class,
                    checkExistResourceGroup(resourceGroup.getResourceGroupId()));
            // 保存商户下联合商户资源组授权范围变化日志
            unionStoreChangeApiService.save(unionEnterpriseAuthDTO.getOwnEnterpriseId(), unionEnterpriseAuthDTO.getUnionEnterpriseId());
            //通知业务方数据同步联合调整

            TransmitDataDTO transmitDataDTO = TransmitDataHolder.get();
            syncResourceAuthChangeMQ(unionEnterpriseAuthDTO.getOwnEnterpriseId(), unionEnterpriseAuthDTO.getUnionEnterpriseId(),
                    resourceGroup, oldResourceGroup, 2, JSON.toJSONString(transmitDataDTO));
        }

        resourceGroupService.editResourceGroup(resourceGroup);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> syncResourceAuthChangeMQ(Integer ownEnterpriseId, Integer unionEnterpriseId,
                                                          ResourceGroupDTO newResourceGroup, ResourceGroupDTO oldResourceGroup, Integer resourceGroupChangeType, String logParams) {
        UnionEnterpriseAuthResNoticeDTO unionEnterpriseAuthResNoticeDTO = new UnionEnterpriseAuthResNoticeDTO();
        unionEnterpriseAuthResNoticeDTO.setGoodsResourceId(newResourceGroup.getGoodsResourceId());
        unionEnterpriseAuthResNoticeDTO.setStoreResource(newResourceGroup.getStoreResource());
        unionEnterpriseAuthResNoticeDTO.setOwnEnterpriseId(ownEnterpriseId);
        unionEnterpriseAuthResNoticeDTO.setUnionEnterpriseId(unionEnterpriseId);
        unionEnterpriseAuthResNoticeDTO.setResourceGroupChangeType(resourceGroupChangeType);

        unionEnterpriseAuthResNoticeDTO.setLogParams(logParams);
        unionEnterpriseAuthResNoticeDTO.setSmsId(newResourceGroup.getSmsId());
        unionEnterpriseAuthResNoticeDTO.setGoodsResourceConfig(newResourceGroup.getGoodsResourceConfig());

        //门店通知
        if (newResourceGroup.getStoreResource() != null) {
            try {
                storeAuthorizationApiService.authStore(newResourceGroup.getStoreResource().intValue(),unionEnterpriseId, ownEnterpriseId);
            } catch (Exception e) {
                logger.warn("资源组修改授权门店错误:{}", e.getMessage(), e);
            }
        }

        CustomUserAreaDTO customUserAreaDTO = ResourceGroupUtils
                .getResource(ResourceGroupUtils.CUSTOM_RESOURCE, EntityUtil.changeEntityNew(TabSysResourceGroup.class, newResourceGroup));
        unionEnterpriseAuthResNoticeDTO.setCustomAreaDTO(EntityUtil.changeEntityNew(CustomAreaDTO.class, customUserAreaDTO));
        List<UserResourceChangeInfoDTO> userResourceInfoList = new ArrayList<>();


        Map<Integer, List<Long>> oldUserAreaMap = dealUserResourceData(oldResourceGroup);

        if (newResourceGroup.getCustomUser() == 1) {
            Map<Integer, List<Long>> newUserAreaMap = dealUserResourceData(newResourceGroup);
            newUserAreaMap.entrySet().forEach(e -> {
                List<Long> oldAreaList = oldUserAreaMap.get(e.getKey());

                UserResourceChangeInfoDTO userResourceChangeInfoDTO = new UserResourceChangeInfoDTO();
                userResourceChangeInfoDTO.setType(e.getKey());
                if (oldAreaList == null) {
                    userResourceChangeInfoDTO.setAddList(e.getValue());
                } else {
                    List<Long> newAreaList = e.getValue();

                    List<Long> addList = new ArrayList<>();
                    List<Long> deleteList = new ArrayList<>();

                    for (Long newArea : newAreaList) {
                        if (!oldAreaList.contains(newArea)) {
                            //如果新的数据在旧的数据里面不存在了，则说明增加了域
                            addList.add(newArea);
                        }
                    }
                    for (Long oldArea : oldAreaList) {
                        if (!newAreaList.contains(oldArea)) {
                            //如果旧的数据在新的数据里面不存在，则说明被删除了域
                            deleteList.add(oldArea);
                        }
                    }
                    userResourceChangeInfoDTO.setAddList(addList);
                    userResourceChangeInfoDTO.setDeleteList(deleteList);
                }

                userResourceInfoList.add(userResourceChangeInfoDTO);
            });
        } else {
            oldUserAreaMap.entrySet().forEach(e -> {
                UserResourceChangeInfoDTO userResourceChangeInfoDTO = new UserResourceChangeInfoDTO();
                userResourceChangeInfoDTO.setType(e.getKey());
                userResourceChangeInfoDTO.setDeleteList(e.getValue());

                userResourceInfoList.add(userResourceChangeInfoDTO);
            });
        }

        unionEnterpriseAuthResNoticeDTO.setUserResourceInfoList(userResourceInfoList);

        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("联合商户资源授权通知:{}", JSON.toJSONString(unionEnterpriseAuthResNoticeDTO));
            client.sendMessage("syncResourceAuthChangeMQ", JSONObject.toJSONString(unionEnterpriseAuthResNoticeDTO));
        } catch (Exception e) {
            logger.warn("联合商户资源授权路由:{}", e.getMessage(), e);
        }

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<ResourceGroupDTO>> pageResourceGroup(String name, Integer enterpriseId,
                                                                     Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabSysResourceGroup> page = resourceGroupService.pageResourceGroup(name, enterpriseId, pageNum, pageSize);
        Page<ResourceGroupDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, ResourceGroupDTO.class);
        List<ResourceGroupDTO> resourceList = resultPage.getResult();
        if (CollectionUtils.isNotEmpty(resourceList)) {
            //查询授权管理员数量
            List<Integer> resourceIds = resourceList.stream()
                    .map(ResourceGroupDTO::getResourceGroupId)
                    .collect(Collectors.toList());
            Map<Integer, Integer> map = userResourceService.countByResourceIds(enterpriseId, resourceIds);
            for (ResourceGroupDTO resource : resourceList) {
                Integer count = map.get(resource.getResourceGroupId());
                resource.setUserResourceCount(count == null ? 0 : count);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<ResourceGroupDTO>> listResourceGroup(String name, Integer enterpriseId) {
        return ServiceResponse.success(EntityUtil.changeEntityListNew(ResourceGroupDTO.class,
                resourceGroupService.listResourceGroup(name, enterpriseId)));
    }

    @Override
    public ServiceResponse<ResourceGroupDTO> getResourceGroup(Integer resourceGroupId) {
        TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
        }
        ResourceGroupDTO result = EntityUtil.changeEntityNew(ResourceGroupDTO.class,
                record);
        if (StringUtils.isNotBlank(result.getCustomUserData())) {
            result.setCustomUserList(JSON.parseArray(result.getCustomUserData(), ResourceGroupCustomUserDTO.class));
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<ResourceGroupDTO>> listResourceGroup(List<Integer> resourceGroupIdList) {
        List<TabSysResourceGroup> list = resourceGroupService.listResourceGroup(resourceGroupIdList);
        if (CollectionUtils.isEmpty(list)) {
            return ServiceResponse.success(Collections.emptyList());
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(ResourceGroupDTO.class, list));
    }

    @Override
    public ServiceResponse<String> deleteResourceGroup(Integer resourceGroupId) {
        TabSysResourceGroup record = checkExistResourceGroup(resourceGroupId);
        //是否联合授权，如果是，则不允许删除
        if (unionEnterpriseAuthApiService.hasUnionEnterpriseAuth(resourceGroupId).getResult()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "该资源组已授权给其他商户，不能删除");
        }
        //验证是否有管理员使用中
        boolean exist = this.userResourceService.existUserResource(record.getEnterpriseId(), record.getResourceGroupId());
        if (exist) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该资源组正在使用中，不能删除");
        }
        resourceGroupService.deleteResourceGroup(resourceGroupId);
        return ServiceResponse.success(record.getResourceGroupName());
    }

    @Override
    public ServiceResponse<ResourceGroupDataDTO> getResourceGroupByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId));
    }

    @Override
    public ServiceResponse<StoreResourceDTO> getStoreResourceByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId, ResourceGroupUtils.STORE_RESOURCE));
    }

    @Override
    public ServiceResponse<CustomUserAreaDTO> getCustomUserByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId, ResourceGroupUtils.CUSTOM_RESOURCE));
    }

    @Override
    public ServiceResponse<GoodsResourceDTO> getGoodsResourceByUserId(Integer userId) {
        return ServiceResponse.success(ResourceGroupUtils.getResource(userId, ResourceGroupUtils.GOODS_RESOURCE));
    }

    /**
     * 参数验证
     * @param resourceGroup
     * @return null：验证OK
     */
    private void validParam(ResourceGroupDTO resourceGroup) {
        Integer resourceGroupId = resourceGroup.getResourceGroupId();
        if (resourceGroupId != null) {
            TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
            if (record == null) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
            }
        }
        //param valid
        if (StringUtils.isBlank(resourceGroup.getResourceGroupName())) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称不能为空");
        }
        Integer enterpriseId =  resourceGroup.getEnterpriseId();
        if (enterpriseId == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (resourceGroupService.isRepeatResourceGroupName(resourceGroupId, resourceGroup.getResourceGroupName(), enterpriseId)) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称重复");
        }
        Integer customUser = resourceGroup.getCustomUser();
        if (customUser == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "用户域授予标志不能为空");
        }
        //用户域是否授予
        boolean hasAuthCustomUser = customUser == 1;
        String customUserData = resourceGroup.getCustomUserData();
        if (hasAuthCustomUser) {
            if (StringUtils.isBlank(customUserData)) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "用户域授予资源为空，请选择不授予");
            }

            try {
                List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(customUserData, ResourceGroupCustomUserDTO.class);
            } catch (Exception e) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "用户域数据格式有误");
            }
        }
        //微盟订单店铺配置值
        setWeimobStore(resourceGroup);
    }

    private void setWeimobStore(ResourceGroupDTO resourceGroupDTO) {
        Integer ownEnterpriseId = resourceGroupDTO.getEnterpriseId();
        String weimobStoreId = resourceGroupDTO.getWeimobStoreId();
        //授予店铺订单资源
        if (resourceGroupDTO.isAuthWeimobOrder() && StringUtils.isNotBlank(weimobStoreId)) {
            //查询所有的店铺信息
            List<String> wmStoreIdList = Arrays.stream(weimobStoreId.split(",")).collect(Collectors.toList());
            ServiceResponse<List<WmStoreDTO>> response = wmStoreApiService.listWmStore(wmStoreIdList, null);
            if (response.isSuccess()) {
                List<WmStoreDTO> wmStoreList = response.getResult();
                if (CollectionUtils.isNotEmpty(wmStoreList)) {
                    //区分自有商户的店铺和共享店铺，分开存放，避免重复授权（共享店铺不能二次共享授权）
                    StringBuilder weimobStoreIdOwn = new StringBuilder();
                    StringBuilder weimobStoreIdShare = new StringBuilder();
                    for (int i = 0, len = wmStoreList.size(); i < len; i ++) {
                        WmStoreDTO temp = wmStoreList.get(i);
                        if (i == len - 1) {
                            //最后一个，不用加逗号
                            if (ownEnterpriseId.equals(temp.getEnterpriseId())) {
                                weimobStoreIdOwn.append(temp.getWmStoreId());
                            } else {
                                weimobStoreIdShare.append(temp.getWmStoreId());
                            }
                        } else {
                            if (ownEnterpriseId.equals(temp.getEnterpriseId())) {
                                weimobStoreIdOwn.append(temp.getWmStoreId()).append(",");
                            } else {
                                weimobStoreIdShare.append(temp.getWmStoreId()).append(",");
                            }
                        }
                    }
                    //赋值
                    resourceGroupDTO.setWeimobStoreIdOwn(weimobStoreIdOwn.toString());
                    resourceGroupDTO.setWeimobStoreIdShare(weimobStoreIdShare.toString());
                }
            }
        }
    }

    private TabSysResourceGroup checkExistResourceGroup(Integer resourceGroupId) {
        TabSysResourceGroup record = resourceGroupService.getResourceGroup(resourceGroupId);
        if (record == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不存在");
        }
        return record;
    }

    private Map<Integer, List<Long>> dealUserResourceData(ResourceGroupDTO oldResourceGroup) {
        //处理旧的用户域数据，用于比对区分增加/删除域数据
        Map<Integer, List<Long>> oldUserAreaMap = new HashMap<>();
        if (oldResourceGroup != null) {
            if (oldResourceGroup.getCustomUser() == 1) {
                List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(oldResourceGroup.getCustomUserData(), ResourceGroupCustomUserDTO.class);
                Map<Integer, List<ResourceGroupCustomUserDTO>> map = customUserList.stream().collect(Collectors.groupingBy(ResourceGroupCustomUserDTO::getType));
                map.entrySet().forEach(e -> {
                    List<ResourceGroupCustomUserDTO> tempList = e.getValue();
                    List<Long> areaValueList = new ArrayList<>();
                    for (ResourceGroupCustomUserDTO temp : tempList) {
                        String[] areaValueArr = temp.getAreaValue().split(",");
                        for (String areaValue : areaValueArr) {
                            if (org.apache.commons.lang3.StringUtils.isBlank(areaValue)) {
                                continue;
                            }
                            areaValueList.add(Long.valueOf(areaValue));
                        }
                    }
                    oldUserAreaMap.put(e.getKey(), areaValueList);
                });
            }
        }

        //补充类型数据
        if (oldUserAreaMap.get(CuTypeEnum.ACU.getType()) == null) {
            oldUserAreaMap.put(CuTypeEnum.ACU.getType(), new ArrayList<>());
        }
        if (oldUserAreaMap.get(CuTypeEnum.SCU.getType()) == null) {
            oldUserAreaMap.put(CuTypeEnum.SCU.getType(), new ArrayList<>());
        }
        if (oldUserAreaMap.get(CuTypeEnum.MCU.getType()) == null) {
            oldUserAreaMap.put(CuTypeEnum.MCU.getType(), new ArrayList<>());
        }
        return oldUserAreaMap;
    }
}
