package com.gic.auth.utils.qrcode;

import com.gic.auth.entity.TabWechatHandleLog;
import com.gic.auth.service.UserApiService;
import com.gic.auth.service.UserService;
import com.gic.auth.service.WechatHandleLogService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.entity.TabAuditor;
import com.gic.auth.service.AuditorService;

/**
 * 审核员二维码回调
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/5 11:09 AM 
 */
@Component
public class AuditorQrcode implements IQrcode{
    private static Logger logger = LogManager.getLogger(AuditorQrcode.class);
    @Autowired
    private AuditorService auditorService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private UserService userService;
    @Autowired
    private WechatHandleLogService wechatHandleLogService;

    @Override
    public void receiveOpenId(String customParams, String openid) {
        wechatHandleLogService.saveWechatHandleLog(new TabWechatHandleLog()
                .setBusinessModule("获取openId")
                .setBusinessModuleInterface("com.gic.auth.utils.qrcode.AuditorQrcode.receiveOpenId")
                .setParams("openid:" + openid + ";customParams:" + customParams));

        Integer auditorId = Integer.valueOf(customParams);
        TabAuditor auditor = this.auditorService.getAuditor(auditorId);
        if (auditor != null) {
            boolean repeat = this.auditorService.validOpenidIsRepeat(auditor.getEnterpriseId(), openid, auditor.getAuditorId());
            if (repeat) {
                logger.warn("receiveOpenid失败，商户：{}下，存在重复的openid：{}", auditor.getEnterpriseId(), openid);
            } else {
                AuditorDTO auditorDTO = new AuditorDTO();
                auditorDTO.setAuditorId(auditorId);
                auditorDTO.setOpenid(openid);
                auditorDTO.setIsUse(1);
                this.auditorService.update(auditorDTO);

                //同步到管理员,如果是管理管理员
                if (auditor.getUserId() != null) {
                    boolean isRepeat = userService.validOpenidIsRepeat(auditor.getEnterpriseId(), openid, auditor.getUserId());
                    if (!isRepeat) {
                        userApiService.updateUserOpenId(auditor.getUserId(), openid);
                    }
                }
            }
        }
    }
}
