package com.gic.auth.utils.qrcode;

import com.alibaba.fastjson.JSON;
import com.gic.auth.config.Config;
import com.gic.auth.dto.wechat.WechatUserDTO;
import com.gic.auth.entity.TabWechatHandleLog;
import com.gic.auth.service.WechatHandleLogService;
import com.gic.commons.util.EntityUtil;
import com.gic.wechat.business.api.dto.fwh.WeixinUserAuthDTO;
import com.gic.wechat.business.api.service.fwh.WeixinBaseFunService;
import com.gic.wechat.business.api.service.fwh.WeixinUserFunService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.utils.ApplicationContextUtils;
import com.gic.enterprise.constant.QrcodeTypeEnum;
import com.gic.enterprise.dto.QrcodeDTO;
import com.gic.enterprise.service.QrCodeApiService;
/**
 * 二维码工具
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/5 2:02 PM 
 */
public class QrcodeUtils {
    private static Logger logger = LogManager.getLogger(QrcodeUtils.class);

    /**
     * 二维码回调
     * @param qrCodeParam
     * @param openid
     */
    public static void qrCodeCallback(String qrCodeParam, String openid) {
        logger.info("二维码参数：{},openid:{}", qrCodeParam, openid);
        boolean isSuccess = StringUtils.isNotBlank(qrCodeParam) && StringUtils.isNotBlank(openid);
        if (isSuccess) {
            if (!StringUtils.isNumeric(qrCodeParam)) {
                logger.info("管理员二维码回调参数有误");
                return;
            }
            QrCodeApiService qrCodeApiService = ApplicationContextUtils.getBean("qrCodeApiService");
            ServiceResponse<QrcodeDTO> qrCodeResponse = qrCodeApiService.getQrcodeDownload(Integer.parseInt(qrCodeParam));
            if (qrCodeResponse.isSuccess()) {
                QrcodeDTO qrcodeDTO = qrCodeResponse.getResult();
                if (QrcodeTypeEnum.AUDITOR.getCode().equals(qrcodeDTO.getQrCodeType())) {
                    ApplicationContextUtils.getBean(AuditorQrcode.class).receiveOpenId(qrcodeDTO.getCustomParams(), openid);
                } else if (QrcodeTypeEnum.USER.getCode().equals(qrcodeDTO.getQrCodeType())) {
                    ApplicationContextUtils.getBean(UserQrcode.class).receiveOpenId(qrcodeDTO.getCustomParams(), openid);
                }
            }
        }
    }

    /**
     * 查询服务号关注用户信息
     * @param openId
     * @return
     */
    public static WechatUserDTO getMemberInfoByOpenId(String openId) {
        if (StringUtils.isBlank(openId)) {
            return null;
        }
        WeixinUserFunService weixinUserFunService = ApplicationContextUtils.getBean("weixinUserFunService");
        Config config = ApplicationContextUtils.getBean("config");
        ServiceResponse<WeixinUserAuthDTO> response = weixinUserFunService.getMemberInfo(config.getAppkey(), openId);
        if (response.isSuccess()) {
            WeixinUserAuthDTO weixinUserAuthDTO = response.getResult();
            if (weixinUserAuthDTO != null) {
                return EntityUtil.changeEntityNew(WechatUserDTO.class, weixinUserAuthDTO);
            }
            logger.info("openId：{}查询粉丝信息结果:{}", openId, JSON.toJSONString(response));
        }
        logger.info("openId：{}查询粉丝信息失败:{}", openId, response.getMessage());
        return null;
    }

    /**
     * 获取二维码url
     * @param qrCodeType
     * @param serviceId 业务方的主键ID
     * @param enterpriseId
     * @return
     */
    public static String getTempQrCodeUrl(Integer qrCodeType, Integer serviceId, Integer enterpriseId) {
        logger.info("获取临时二维码参数:{}-{}-{}", qrCodeType, serviceId, enterpriseId);
        WeixinBaseFunService weixinBaseFunService = ApplicationContextUtils.getBean("weixinBaseFunService");
        Config config = ApplicationContextUtils.getBean("config");
        //保存二维码
        QrCodeApiService qrCodeApiService = ApplicationContextUtils.getBean("qrCodeApiService");
        QrcodeDTO qrcodeDTO = new QrcodeDTO();
        qrcodeDTO.setType(1);
        qrcodeDTO.setQrCodeType(qrCodeType);
        qrcodeDTO.setEnterpriseId(enterpriseId);
        qrcodeDTO.setCustomParams(String.valueOf(serviceId));
        ServiceResponse<Integer> response = qrCodeApiService.saveQrcodeDownload(qrcodeDTO);
        if (response.isSuccess()) {
            //id用于回调查询使用
            Integer id = response.getResult();

            WechatHandleLogService wechatHandleLogService = ApplicationContextUtils.getBean("wechatHandleLogService");
            ServiceResponse<String> serviceResponse = weixinBaseFunService.getTemporaryQRcodeURL(config.getAppkey(), id.toString(), 300);

            wechatHandleLogService.saveWechatHandleLog(new TabWechatHandleLog()
                    .setBusinessModule("获取二维码")
                    .setBusinessModuleInterface("com.gic.auth.utils.qrcode.QrcodeUtils.getTempQrCodeUrl")
                    .setParams("appkey:" + config.getAppkey() + ";sceneStr:" + id)
                    .setResultMessage(JSON.toJSONString(serviceResponse)));
            if (serviceResponse.isSuccess()) {
                logger.info("临时带参二维码生成成功:{}", serviceResponse.getResult());
                return serviceResponse.getResult();
            } else {
                logger.warn("临时带参二维码生成失败:{}", serviceResponse.getMessage());
                return null;
            }
        }
        return null;
    }

}
