package com.gic.auth.utils.resourcegroup;

import com.gic.auth.dto.*;
import com.gic.auth.entity.TabSysResourceGroup;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.service.ResourceGroupService;
import com.gic.auth.service.UserResourceService;
import com.gic.auth.service.UserService;
import com.gic.auth.utils.ApplicationContextUtils;
import org.apache.commons.collections.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 资源组工具类
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/12 2:31 PM 
 */
public class ResourceGroupUtils {
    private static Map<Integer, IResourceGroupData> map = new HashMap<>(8);
    /**
     * 用户域
     */
    public static Integer CUSTOM_RESOURCE = 1;
    /**
     * 门店
     */
    public static Integer STORE_RESOURCE = 2;
    /**
     * 商品
     */
    public static Integer GOODS_RESOURCE = 3;

    static {
        map.put(CUSTOM_RESOURCE, new CustomResourceData());
        map.put(STORE_RESOURCE, new StoreResourceData());
        map.put(GOODS_RESOURCE, new GoodsResourceData());
    }

    /**
     * 查询资源组资源
     * @param userId
     * @param resourceGroupType 1：用户域 2：门店资源 3：商品
     * @param <T>
     * @return
     */
    public static <T> T getResource(Integer userId, Integer resourceGroupType) {
        UserService userService = ApplicationContextUtils.getBean("userService");
        TabSysUser tabSysUser = userService.getUserById(userId);
        if (tabSysUser == null) {
            return map.get(resourceGroupType).getResourceDataWhenNoExist();
        }
        if (tabSysUser.getSuperAdmin() == 1) {
            //超管
            return map.get(resourceGroupType).getResourceDataForAdmin();
        }
        TabSysResourceGroup tabSysResourceGroup = getByUserId(userId);
        if (tabSysResourceGroup == null) {
            return map.get(resourceGroupType).getResourceDataWhenNoExist();
        }
        return map.get(resourceGroupType).getResourceData(tabSysResourceGroup);
    }

    public static <T> T getResource(Integer resourceGroupType, TabSysResourceGroup tabSysResourceGroup) {
        return map.get(resourceGroupType).getResourceData(tabSysResourceGroup);
    }

    /**
     * 查询全部资源
     * @param userId
     * @return
     */
    public static ResourceGroupDataDTO getResource(Integer userId) {
        CustomUserAreaDTO customUserAreaDTO = getResource(userId, CUSTOM_RESOURCE);
        StoreResourceDTO storeResourceDTO = getResource(userId, STORE_RESOURCE);
        GoodsResourceDTO goodsResourceDTO = getResource(userId, GOODS_RESOURCE);
        return new ResourceGroupDataDTO()
                .setCustomUserAreaDTO(customUserAreaDTO)
                .setGoodsResourceDTO(goodsResourceDTO)
                .setStoreResourceDTO(storeResourceDTO);
    }

    private static TabSysResourceGroup getByUserId(Integer userId) {
        UserResourceService userResourceService = ApplicationContextUtils.getBean(UserResourceService.class);
        ResourceGroupService resourceGroupService = ApplicationContextUtils.getBean("resourceGroupService");
        List<UserResourceDTO> userResourceList = userResourceService.listUserResourceByUserId(userId);
        if (CollectionUtils.isEmpty(userResourceList)) {
            //查不到管理员的资源组数据，历史数据导致
            return null;
        }
        //资源组单选
        Integer resourceGroupId = userResourceList.get(0).getResourceId();
        TabSysResourceGroup resourceGroup = resourceGroupService.getResourceGroup(resourceGroupId);
        return resourceGroup;
    }
}
