package com.gic.auth.web.controller;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuItemApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.utils.EnterpriseBasicAppUtils;
import com.gic.auth.web.vo.LogListVO;
import com.gic.auth.web.vo.UserOperationItemAuthVO;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.log.api.dto.LogSearchDTO;
import com.gic.log.api.dto.SystemSetLogDTO;
import com.gic.log.api.service.LogApiService;
/**
 *
 * @ClassName: MenuController
 * @Description:
 * @author zhiwj
 * @date 2020-09-10 10:01
 */
@RestController
public class MenuController {

    private final static Logger LOGGER = LogManager.getLogger(MenuController.class);
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private MenuItemApiService menuItemApiService;
    @Autowired
    private LogApiService logApiService;
    @Autowired
    private UserApiService userApiService;

    /**
     * 查询登录用户的gic菜单
     * @Title: loginUserMenu 
     * @Description:
     * @author guojuxing
     * @param currentMenu 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("login-user-menu")
    public RestResponse loginUserMenu(Integer currentMenu) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        Integer userId = userDetail.getUserId();
        Integer enterpriseId = userDetail.getEnterpriseId();
        ServiceResponse<List<MenuDTO>> userMenu = this.menuApiService.getUserMenuOfGic(enterpriseId, userId,
                currentMenu, UserDetailUtils.getUserDetail().getEnterpriseInfo().getVersionCode());
        if (userMenu.isSuccess()) {
            return RestResponse.success(userMenu.getResult());
        }
        return EnterpriseRestResponse.failure(userMenu);
    }

    /**
     * 查询登录用户的应用的菜单
     * @Title: loginUserMenu 
     * @Description:
     * @author guojuxing
     * @param currentMenu
     * @param appId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("login-user-menu-of-app")
    public RestResponse loginUserMenu(Integer currentMenu, String appId, String menuCode) {
        LOGGER.info("查询登录用户的应用的菜单参数：{}-{}-{}", currentMenu, appId, menuCode);
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        appId = getAppId(appId, menuCode);
        ServiceResponse<List<MenuDTO>> userMenu = this.menuApiService.getUserMenuOfApp(enterpriseId, userId,
                currentMenu, appId);
        if (userMenu.isSuccess()) {
            return RestResponse.success(userMenu.getResult());
        }
        return EnterpriseRestResponse.failure(userMenu);
    }

    /**
     * 根据应用code查询应用ID
     * @Title: getAppIdByMenuCode 
     * @Description:
     * @author guojuxing
     * @param menuCode 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("get-app-by-code")
    public RestResponse getAppIdByMenuCode(String menuCode) {
        return RestResponse.success(getAppId(null, menuCode));
    }

    /**
     * gic菜单转到某一个应用下面
     * @param menuCode
     * @return
     */
    @IgnoreLogin
    @RequestMapping("sync-menu-to-pro")
    public RestResponse syncMenuToPro(String menuCode) {
        return RestResponse.success(menuApiService.syncProductMenu(menuCode));
    }

    @IgnoreLogin
    @HeaderSignIgnore
    @RequestMapping("switch-gic-menu-to-app")
    public RestResponse switchGicMenuToApp(String gicMenuCode, String appMenuCode, Boolean appToGic) {
        ServiceResponse response;
        if (appToGic != null && appToGic) {
            response = menuApiService.switchAppMenuToGic(gicMenuCode, appMenuCode);
        } else {
            response = menuApiService.switchGicMenuToApp(gicMenuCode, appMenuCode);
        }
        return ResultControllerUtils.commonResult(response);
    }

    /**
     * 登陆用户没有权限的操作项列表数据（gic）
     * @Title: loginUserOperationItem 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("login-user-operation-item")
    public RestResponse loginUserOperationItem() {
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        return ResultControllerUtils.commonResult(
                this.menuApiService.getNoAuthOperationItemOfGic(userId,
                        UserDetailUtils.getUserDetail().getEnterpriseInfo().getVersionCode()),
                UserOperationItemAuthVO.class);
    }

    /**
     * 登陆用户没有权限的操作项列表数据（app）
     * @Title: loginUserOperationItemOfApp 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("login-user-operation-item-of-app")
    public RestResponse loginUserOperationItemOfApp(String appId, String menuCode) {
        LOGGER.info("登陆用户没有权限的操作项列表数据：{}-{}", appId, menuCode);
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        appId = getAppId(appId, menuCode);
        ServiceResponse<List<MenuDTO>> menuList = this.menuApiService.getNoAuthOperationItemOfApp(userId, "");
        List<UserOperationItemAuthVO> resultList = new ArrayList<>();
        if (menuList.isSuccess()) {
            List<MenuDTO> list = menuList.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (MenuDTO menuDTO : list) {
                    if (menuDTO.getProject().equals(appId)) {
                        resultList.add(EntityUtil.changeEntityNew(UserOperationItemAuthVO.class, menuDTO));
                    }
                }
            }
        }
        return RestResponse.success(resultList);
    }

    /**
     * 获取的页面树
     * @Title: listMenuPageTree 
     * @Description:
      * @author guojuxing
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-log-menu-tree")
    public RestResponse listMenuPageTree() {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        //权限控制
        ServiceResponse<List<MenuDTO>> result = menuApiService
                .getUserMenuOfGicNotTree(userDetail.getEnterpriseId(),
                userDetail.getUserId(), null, userDetail.getEnterpriseInfo().getVersionCode());
        if (result.isSuccess()) {
            return RestResponse
                    .success(changePageListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/list-log-app-menu-tree")
    public RestResponse listMenuAppPageTree() {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        boolean isSuperAdmin = userDetail.getUserInfo().getSuperAdmin().intValue() == 1;
        ServiceResponse<List<MenuDTO>> result;
        if (isSuperAdmin) {
            result = menuApiService.listAppPageOfHasBuy(UserDetailUtils.getUserDetail().getEnterpriseId(), userDetail.getUserId());
        } else {
            //权限控制
            result = menuApiService.getUserMenuOfAppNotTree(userDetail.getEnterpriseId(), userDetail.getUserId(), null,
                    null);
        }
        if (result.isSuccess()) {
            //过滤商户基础应用
            List<MenuDTO> appMenuList = result.getResult();
            if (CollectionUtils.isNotEmpty(appMenuList)) {
                appMenuList = appMenuList.stream()
                        .filter(e -> !EnterpriseBasicAppUtils.ENTERPRISE_BASIC_APP_CODE.equals(e.getMenuCode()))
                        .collect(Collectors.toList());
            }
            return RestResponse
                    .success(changePageListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, appMenuList));
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 分页查询操作日志
     * @Title: listOperationLog 
     * @Description:
      * @author guojuxing
     * @param startTime
     * @param endTime
     * @param menuId
     * @param search 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-log")
    public RestResponse listOperationLog(String startTime, String endTime, Integer menuId, String search, PageQO pageQO)
            throws ParseException {

        UserDetail userDetail = UserDetailUtils.getUserDetail();
        Integer enterpriseId = userDetail.getEnterpriseId();
        //是否是超级管理员
        boolean isSuperAdmin = userDetail.getUserInfo().getSuperAdmin().intValue() == 1;

        LogSearchDTO searchParam = new LogSearchDTO();
        searchParam.setEnterpriseId(Long.valueOf(enterpriseId));
        searchParam.setInterfaceName(search);
        if (menuId != null) {
            searchParam.setRelationId(Long.valueOf(menuId));
        } else {
            //权限控制
            List<Long> relationIdList = new ArrayList<>();
            //权限控制
            ServiceResponse<List<MenuDTO>> gicResult = menuApiService.getUserMenuOfGicNotTree(
                    userDetail.getEnterpriseId(), userDetail.getUserId(), null,
                    userDetail.getEnterpriseInfo().getVersionCode());
            if (gicResult.isSuccess()) {
                List<MenuDTO> gicMenuList = gicResult.getResult();
                relationIdList.addAll(Optional.ofNullable(gicMenuList).orElse(new ArrayList<>()).stream()
                        .mapToLong(e -> e.getMenuId()).boxed().collect(Collectors.toList()));
            }
            ServiceResponse<List<MenuDTO>> appResult;
            if (isSuperAdmin) {
                appResult = menuApiService.listAppPageOfHasBuy(UserDetailUtils.getUserDetail().getEnterpriseId(), userDetail.getUserId());
            } else {
                //权限控制
                appResult = menuApiService.getUserMenuOfAppNotTree(userDetail.getEnterpriseId(), userDetail.getUserId(),
                        null, null);
            }
            if (appResult.isSuccess()) {
                List<MenuDTO> appMenuList = appResult.getResult();
                relationIdList.addAll(Optional.ofNullable(appMenuList).orElse(new ArrayList<>()).stream()
                        .mapToLong(e -> e.getMenuId()).boxed().collect(Collectors.toList()));
            }
            searchParam.setRelationIdList(relationIdList);
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if (StringUtils.isNotBlank(startTime)) {
            startTime = startTime + " 00:00:00";
            searchParam.setLogTimeBegin(sdf.parse(startTime));
        }
        if (StringUtils.isNotBlank(endTime)) {
            endTime = endTime + " 23:59:59";
            searchParam.setLogTimeEnd(sdf.parse(endTime));
        }
        //过滤登录人员类型：运营人员/gic管理员
        searchParam.setLoginType(userDetail.getUserInfo().getLoginType().toString());

        ServiceResponse<Page<SystemSetLogDTO>> response = logApiService.listSystemSetLog(pageQO.getPageSize(),
                pageQO.getCurrentPage(), searchParam);
        if (response.isSuccess()) {
            Page page = (Page) response.getResult();
            if (page == null) {
                return RestResponse.success();
            }
            List<SystemSetLogDTO> responseList = page.getResult();
            if (CollectionUtils.isNotEmpty(responseList)) {
                List<LogListVO> resultList = responseList.stream().map(e -> {
                    LogListVO temp = EntityUtil.changeEntityNew(LogListVO.class, e);
                    temp.setNationCodeFrom(e.getRemark1());
                    temp.setPhoneFrom(e.getRemark2());
                    return temp;
                }).collect(Collectors.toList());
                page.setResult(resultList);
                return RestResponse.success(page);
            }
            return RestResponse.success();
        } else {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
    }

    public static List<Map<String, Object>> changePageListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (int j = 0, length = list.size(); j < length; j++) {
                MenuDTO menuDTO = list.get(j);
                if (storeGroupId == menuDTO.getParentId() && menuDTO.getLevel().intValue() <= 2) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("menuCode", menuDTO.getMenuCode());
                    data.put("parentCode", menuDTO.getParentCode());
                    data.put("parentId", menuDTO.getParentId());
                    List<Map<String, Object>> children = changePageListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

    private String getAppId(String appId, String menuCode) {
        if (StringUtils.isBlank(appId)) {
            //根据menuCode去查询对应的appId
            ServiceResponse<MenuDTO> resp = menuApiService.getMenuByMenuCode(menuCode);
            if (resp.isSuccess()) {
                MenuDTO tempDTO = resp.getResult();
                appId = tempDTO.getProject();
            }
        }
        return appId;
    }

}
