package com.gic.auth.web.controller;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.auth.dto.AuthCodeDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.UserApiService;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.web.vo.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.enums.ShopTypeEnum;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.service.sms.SmsApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.open.api.dto.EnterpriseServeInfoDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.EnterpriseLicenseApiService;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/29 9:09
 */
@RestController
public class ResourceController {
    
    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;
    @Autowired
    private SmsApiService smsApiService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private EnterpriseLicenseApiService enterpriseLicenseApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private WmStoreApiService wmStoreApiService;

    private static final String UNION_ENTERPRISE_AUTH_SMS_CODE = "GICSHHT001";

    @RequestMapping("/list-wm-store")
    public RestResponse listWmStore() {
        List<WmStoreVO> result = new ArrayList<>();
        List<Integer> enterpriseIdList = new ArrayList<>();
        enterpriseIdList.add(UserDetailUtils.getUserDetail().getEnterpriseId());
        //自有店铺查询
        ServiceResponse<List<WmStoreDTO>> listResponse = wmStoreApiService.listWmStore(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (listResponse.isSuccess()) {
            List<WmStoreDTO> list = listResponse.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (WmStoreDTO temp : list) {
                    WmStoreVO vo = EntityUtil.changeEntityNew(WmStoreVO.class, temp);
                    vo.setOwnerFlag(0);
                    vo.setOwnerEntId(temp.getEnterpriseId());
                    result.add(vo);
                }
            }
        }
        //联合商户授权店铺
        ServiceResponse<List<UnionEnterpriseAuthDTO>> unionResponse = unionEnterpriseAuthApiService
                .listOwnEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (unionResponse.isSuccess()) {
            List<UnionEnterpriseAuthDTO> unionList = unionResponse.getResult();
            if (CollectionUtils.isNotEmpty(unionList)) {
                //为了获取商户名称
                unionList.forEach(e -> {
                    enterpriseIdList.add(e.getOwnEnterpriseId());
                });
                List<Integer> resourceIdList = unionList.stream()
                        .filter(e -> e.getResourceGroupId() != null)
                        .map(e -> e.getResourceGroupId()).collect(Collectors.toList());
                ServiceResponse<List<ResourceGroupDTO>> resourceResponse = resourceGroupApiService.listResourceGroup(resourceIdList);
                if (resourceResponse.isSuccess()) {
                    List<ResourceGroupDTO> resourceList = resourceResponse.getResult();
                    if (CollectionUtils.isNotEmpty(resourceList)) {
                        List<String> wmStoreIdList = new ArrayList<>();
                        for (ResourceGroupDTO resource : resourceList) {
                            String weimobStoreIdOwn = resource.getWeimobStoreIdOwn();
                            if (resource.isAuthWeimobOrder() && StringUtils.isNotBlank(weimobStoreIdOwn)) {
                                wmStoreIdList.addAll(Arrays.asList(weimobStoreIdOwn.split(",")).stream().collect(Collectors.toList()));
                            }
                        }
                        //查询
                        ServiceResponse<List<WmStoreDTO>> wmStoreResponse = wmStoreApiService.listWmStore(wmStoreIdList, null);
                        if (wmStoreResponse.isSuccess()) {
                            List<WmStoreDTO> wmStoreList = wmStoreResponse.getResult();
                            if (CollectionUtils.isNotEmpty(wmStoreList)) {
                                for (WmStoreDTO temp : wmStoreList) {
                                    WmStoreVO vo = EntityUtil.changeEntityNew(WmStoreVO.class, temp);
                                    vo.setOwnerFlag(1);
                                    vo.setOwnerEntId(temp.getEnterpriseId());
                                    result.add(vo);
                                }
                            }
                        }
                    }
                }
            }
        }
        //商户数据
        Map<Integer, String> enterpriseIdMapName = new HashMap<>();
        ServiceResponse<List<EnterpriseDTO>> enterpriseResponse = enterpriseApiService.listEnterpriseByIds(enterpriseIdList);
        if (enterpriseResponse.isSuccess()) {
            List<EnterpriseDTO> enterpriseList = enterpriseResponse.getResult();
            if (CollectionUtils.isNotEmpty(enterpriseList)) {
                enterpriseList.forEach(e -> enterpriseIdMapName.put(e.getEnterpriseId(), e.getEnterpriseName()));
            }
        }
        //数据处理
        if (CollectionUtils.isNotEmpty(result)) {
            for (WmStoreVO vo : result) {
                vo.setOwnerEntName(enterpriseIdMapName.get(vo.getOwnerEntId()));
            }
        }

        return RestResponse.success(result);
    }

    @RequestMapping("/list-sms-sign")
    public RestResponse listSmsSign(String search) {
        ServiceResponse<List<EnterpriseSmsSignDTO>> response = smsApiService
                .listSignByEnterprise(UserDetailUtils.getUserDetail().getEnterpriseId(), search);
        if (response.isSuccess()) {
            return RestResponse.success(EntityUtil.changeEntityListByJSON(EnterpriseSmsSignVO.class, response.getResult()));
        } else {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
    }

    @RequestMapping("/save-resource")
    public RestResponse saveResourceNew(ResourceGroupDTO resourceGroup) {
        resourceGroup.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(resourceGroupApiService.saveResourceGroup(resourceGroup),
                OperationResultUtils.LOG_NEW + "资源组", resourceGroup.getResourceGroupName());
    }

    @RequestMapping("/edit-resource")
    public RestResponse editResourceNew(ResourceGroupDTO resourceGroup) {
        resourceGroup.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(resourceGroupApiService.editResourceGroup(resourceGroup),
                OperationResultUtils.LOG_EDIT + "资源组", resourceGroup.getResourceGroupName());
    }

    /**
     * 是否授权给联合商户，如果是，则需要发短信给联合商户超管
     * @param resourceGroupId
     * @return
     */
    @RequestMapping("/has-auth-union-enterprise")
    public RestResponse hasAuthUnionEnterprise(Integer resourceGroupId) {
        ServiceResponse<UnionEnterpriseAuthDTO> response = unionEnterpriseAuthApiService.getUnionEnterpriseAuthByResourceGroupId(resourceGroupId);
        if (response.isSuccess()) {
            UnionEnterpriseAuthDTO dto = response.getResult();
            if (dto != null) {
                UnionEnterpriseAuthVO vo = EntityUtil.changeEntityNew(UnionEnterpriseAuthVO.class, dto);
                ServiceResponse<EnterpriseDTO> enterpriseResponse = enterpriseApiService.getEnterpriseById(vo.getUnionEnterpriseId());
                vo.setUnionEnterpriseName(enterpriseResponse.getResult().getEnterpriseName());
                return RestResponse.success(vo);
            } else {
                return RestResponse.success();
            }
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }

    @RequestMapping("send-auth-code-to-union-enterprise")
    public RestResponse sendAuthCode(Integer enterpriseId) {
        EnterpriseDTO unionEnterprise = getEnterprise(enterpriseId);

        ServiceResponse<UserDTO> unionUserRes = userApiService.getUserByEnterpriseId(enterpriseId);
        UserDTO unionUser = unionUserRes.getResult();

        UserDetail userDetail = UserDetailUtils.getUserDetail();

        //验证码生成
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //生成6位数字验证码
        String authCode = CreateRandomUtils.generateNumberStr(6);
        authCodeDTO.setAuthCode(authCode);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.MINUTE, 2);
        authCodeDTO.setExpirationTime(calendar.getTime());
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] { unionEnterprise.getEnterpriseName(),
                    userDetail.getEnterpriseInfo().getEnterpriseName(),
                    authCode,
                    userDetail.getEnterpriseInfo().getEnterpriseName()};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(UNION_ENTERPRISE_AUTH_SMS_CODE,
                    unionEnterprise.getEnterpriseId(),
                    unionUser.getPhoneAreaCode(),
                    unionUser.getPhoneNumber(),
                    smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/valid-auth-code")
    public RestResponse validAuthCode(String authCode, Integer authCodeId) {
        ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(authCodeId, authCode);
        if (!authCodeResult.isSuccess()) {
            return RestResponse.failure(authCodeResult.getCode(), authCodeResult.getMessage());
        }
        return RestResponse.success();
    }

    @RequestMapping("/del-resource")
    public RestResponse delResourceNew(Integer resourceGroupId) {
        return OperationResultUtils.operationResult(resourceGroupApiService.deleteResourceGroup(resourceGroupId),
                OperationResultUtils.LOG_DELETE + "资源组");
    }

    @RequestMapping("/list-resource")
    public RestResponse listResourceNew(String name, Integer currentPage, Integer pageSize) {
        return ResultControllerUtils.commonPageResult(resourceGroupApiService
                .pageResourceGroup(name, UserDetailUtils.getUserDetail().getEnterpriseId(), currentPage, pageSize), ResourceGroupListVO.class);
    }

    @RequestMapping("/get-resource")
    public RestResponse getResourceNew(Integer resourceGroupId) {
        return ResultControllerUtils.commonResultOne(resourceGroupApiService.getResourceGroup(resourceGroupId), ResourceGroupDetailVO.class);
    }

    @RequestMapping("/channel-resource")
    public RestResponse channelResource() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<OwnerMemberCardDTO> memberCardList = memberCardApiService.getAll(enterpriseId).getResult();
        List<AppletsConfigSimpleOutputDTO> appletList = appletsConfigApiService.listValidAppletConfigAppId(enterpriseId)
                .getResult();
        List<ServiceConfigSimpleOutputDTO> serviceList = serviceConfigApiService
                .listValidServiceConfigAppId(enterpriseId).getResult();
        Map<String, Object> map = new HashMap<>(4);
        map.put("memberCardList", EntityUtil.changeEntityListByJSON(OwnerMemberCardVO.class, memberCardList));
        map.put("appletList", EntityUtil.changeEntityListByJSON(AppletsConfigSimpleOutputVO.class, appletList));
        map.put("serviceList", EntityUtil.changeEntityListByJSON(ServiceConfigSimpleOutputVO.class, serviceList));
        return RestResponse.success(map);
    }

    @RequestMapping("/get-cu-type")
    public RestResponse cuTypeEnum() {
        return RestResponse.success(Arrays.stream(CuTypeEnum.values()).collect(Collectors.toMap(CuTypeEnum::getType, CuTypeEnum::getDesc)));
    }

    @RequestMapping("/gic-shop-resource")
    public RestResponse gicShopResource() {
        ServiceResponse<List<ShopDTO>> serviceResponse = shopApiService.getAllShopByEnterpriseIdAndName(UserDetailUtils.getUserDetail().getEnterpriseId(), null, ShopTypeEnum.MALL_SHOP.getCode());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/single-version")
    public RestResponse isSingle() {
        ServiceResponse<EnterpriseServeInfoDTO> serveInfo = enterpriseLicenseApiService.getServeInfo(UserDetailUtils.getUserDetail().getEnterpriseId(), "gic");
        return RestResponse.success("basic".equals(serveInfo.getResult().getVersionCode()));
    }

    private EnterpriseDTO getEnterprise(Integer enterpriseId) {
        //查询联合商户信息
        ServiceResponse<EnterpriseDTO> enterpriseResponse = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResponse.isSuccess()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "商户不存在");
        }
        return enterpriseResponse.getResult();
    }
 }
